import 'package:appier_flutter/appier_flutter.dart';
import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  TestWidgetsFlutterBinding.ensureInitialized();

  group('AppierFlutter', () {
    const _channel = MethodChannel('appier_flutter');
    const _appId = 'c9a84d99960e9837db97';
    const _defaultResponses = <String, dynamic>{
      'configure': null,
      'others': true
    };

    final _exception = PlatformException(code: 'ILLEGAL_ARGUMENTS');
    final _log = <MethodCall>[];
    final _configureMethodCall = isMethodCall('configure', arguments: {
      'appId': _appId,
      'senderId': null,
      'appGroup': null,
      'isDev': false,
      'version': '2.5.0-dev2',
    });

    late Map<String, dynamic> _responses;

    setUp(() {
      _responses = Map<String, dynamic>.from(_defaultResponses);
      _channel.setMockMethodCallHandler((methodCall) {
        _log.add(methodCall);
        final dynamic response =
            _responses[methodCall.method] ?? _responses['others'];
        return Future<dynamic>.value(response);
      });
      _log.clear();
    });

    tearDown(() {
      _log.clear();
      _channel.setMockMethodCallHandler(null);
    });

    group('configure', () {
      test('does not invoke native method when appId is empty', () async {
        await AppierFlutter.configure('');
        expect(_log, <Matcher>[]);
      });

      test('invokes native method', () async {
        await AppierFlutter.configure(_appId);
        expect(_log, [_configureMethodCall]);
      });

      test('fails invoking method', () async {
        _responses['configure'] = _exception;

        await AppierFlutter.configure(_appId);
        expect(_log, [_configureMethodCall]);
      });
    });

    group('after configuration', () {
      setUp(() async {
        await AppierFlutter.configure(_appId);
      });

      group('setUniversalLinkDomains', () {
        test('invokes native method', () async {
          await AppierFlutter.setUniversalLinkDomains(['example.com']);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setUniversalLinkDomains', arguments: {
              'linkDomains': ['example.com']
            }),
          ]);
        });

        test('invokes native method with empty string', () async {
          await AppierFlutter.setUniversalLinkDomains([]);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setUniversalLinkDomains',
                arguments: {'linkDomains': <String>[]}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setUniversalLinkDomains([]);
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('logEvent', () {
        test('does not invoke native method when event name is empty',
            () async {
          await AppierFlutter.logEvent('');
          expect(_log, [_configureMethodCall]);
        });

        test('invokes native method', () async {
          await AppierFlutter.logEvent('cart_add');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('logEvent', arguments: {
              'eventName': 'cart_add',
              'parameters': null,
              'vts': null,
              'vtsCurr': null,
            }),
          ]);
        });

        test('invokes native method with purchase product event', () async {
          await AppierFlutter.logEvent(
            'product_purchased',
            parameters: <String, dynamic>{
              'param1': 2,
              'param2': null,
              'param3': 'appier',
            },
            vts: 100,
            vtsCurr: 'USD',
          );
          expect(_log, [
            _configureMethodCall,
            isMethodCall('logEvent', arguments: {
              'eventName': 'product_purchased',
              'parameters': {
                'param1': 2,
                'param2': null,
                'param3': 'appier',
              },
              'vts': 100,
              'vtsCurr': 'USD',
            }),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.logEvent('cart_add');
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('flush', () {
        test('invokes native method', () async {
          await AppierFlutter.flush();
          expect(_log, [
            _configureMethodCall,
            isMethodCall('flush', arguments: <String, dynamic>{}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.flush();
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setUserId', () {
        test('invokes native method', () async {
          await AppierFlutter.setUserId('2031203056');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setUserId', arguments: {'userId': '2031203056'}),
          ]);
        });

        test('invokes native method with empty string', () async {
          await AppierFlutter.setUserId('');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setUserId', arguments: {'userId': ''}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setUserId('2031203056');
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setName', () {
        test('invokes native method', () async {
          await AppierFlutter.setName('yujiosaka');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setName', arguments: {'name': 'yujiosaka'}),
          ]);
        });

        test('invokes native method with empty string', () async {
          await AppierFlutter.setName('');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setName', arguments: {'name': ''}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setName('yujiosaka');
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setFirstName', () {
        test('invokes native method', () async {
          await AppierFlutter.setFirstName('Yuji');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setFirstName', arguments: {'firstName': 'Yuji'}),
          ]);
        });

        test('invokes native method with empty string', () async {
          await AppierFlutter.setFirstName('');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setFirstName', arguments: {'firstName': ''}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setFirstName('Yuji');
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setLastName', () {
        test('invokes native method', () async {
          await AppierFlutter.setLastName('Isobe');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setLastName', arguments: {'lastName': 'Isobe'}),
          ]);
        });

        test('invokes native method with empty string', () async {
          await AppierFlutter.setLastName('');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setLastName', arguments: {'lastName': ''}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setLastName('Isobe');
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setCity', () {
        test('invokes native method', () async {
          await AppierFlutter.setCity('Tokyo');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setCity', arguments: {'city': 'Tokyo'}),
          ]);
        });

        test('invokes native method with empty string', () async {
          await AppierFlutter.setCity('');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setCity', arguments: {'city': ''}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setCity('Tokyo');
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setEmail', () {
        test('invokes native method', () async {
          await AppierFlutter.setEmail('yuji.isobe@appier.com');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setEmail',
                arguments: {'email': 'yuji.isobe@appier.com'}),
          ]);
        });

        test('invokes native method with empty string', () async {
          await AppierFlutter.setEmail('');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setEmail', arguments: {'email': ''}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setEmail('yuji.isobe@appier.com');
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setPhoneNumber', () {
        test('invokes native method', () async {
          await AppierFlutter.setPhoneNumber('000-0000-0000');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setPhoneNumber',
                arguments: {'phoneNo': '000-0000-0000'}),
          ]);
        });

        test('invokes native method with empty string', () async {
          await AppierFlutter.setPhoneNumber('');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setPhoneNumber', arguments: {'phoneNo': ''}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setPhoneNumber('000-0000-0000');
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setDayOfBirth', () {
        test('invokes native method', () async {
          await AppierFlutter.setDayOfBirth(6);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setDayOfBirth', arguments: {'day': 6}),
          ]);
        });

        test('invokes native method with negative number', () async {
          await AppierFlutter.setDayOfBirth(-1);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setDayOfBirth', arguments: {'day': -1}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setDayOfBirth(6);
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setMonthOfBirth', () {
        test('invokes native method', () async {
          await AppierFlutter.setMonthOfBirth(6);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setMonthOfBirth', arguments: {'month': 6}),
          ]);
        });

        test('invokes native method with negative number', () async {
          await AppierFlutter.setMonthOfBirth(-1);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setMonthOfBirth', arguments: {'month': -1}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setMonthOfBirth(6);
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setYearOfBirth', () {
        test('invokes native method', () async {
          await AppierFlutter.setYearOfBirth(1986);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setYearOfBirth', arguments: {'year': 1986}),
          ]);
        });

        test('invokes native method with negative number', () async {
          await AppierFlutter.setYearOfBirth(-1);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setYearOfBirth', arguments: {'year': -1}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setYearOfBirth(1986);
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setCustomKey', () {
        test('does not invoke native method when key is empty', () async {
          await AppierFlutter.setCustomKey('', 'value');
          expect(_log, [_configureMethodCall]);
        });

        test('invokes native method when null value', () async {
          await AppierFlutter.setCustomKey('key', null);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setCustomKey',
                arguments: {'key': 'key', 'value': null}),
          ]);
        });

        test('invokes native method with string value', () async {
          await AppierFlutter.setCustomKey('key', 'value');
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setCustomKey',
                arguments: {'key': 'key', 'value': 'value'}),
          ]);
        });

        test('invokes native method with int value', () async {
          await AppierFlutter.setCustomKey('key', 1);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setCustomKey', arguments: {'key': 'key', 'value': 1}),
          ]);
        });

        test('invokes native method with double value', () async {
          await AppierFlutter.setCustomKey('key', 1.1);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setCustomKey',
                arguments: {'key': 'key', 'value': 1.1}),
          ]);
        });

        test('invokes native method with boolean value', () async {
          await AppierFlutter.setCustomKey('key', true);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setCustomKey',
                arguments: {'key': 'key', 'value': true}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setCustomKey('key', 'value');
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setAttributionWindow', () {
        test('invokes native method', () async {
          await AppierFlutter.setAttributionWindow(1);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setAttributionWindow', arguments: {'seconds': 1}),
          ]);
        });

        test('invokes native method with negative number', () async {
          await AppierFlutter.setAttributionWindow(-1);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setAttributionWindow', arguments: {'seconds': -1}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setAttributionWindow(1);
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });

      group('setClickAttributionWindow', () {
        test('invokes native method', () async {
          await AppierFlutter.setClickAttributionWindow(1);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setClickAttributionWindow',
                arguments: {'seconds': 1}),
          ]);
        });

        test('invokes native method with negative number', () async {
          await AppierFlutter.setClickAttributionWindow(-1);
          expect(_log, [
            _configureMethodCall,
            isMethodCall('setClickAttributionWindow',
                arguments: {'seconds': -1}),
          ]);
        });

        test('fails invoking native method', () async {
          _responses['others'] = _exception;

          try {
            await AppierFlutter.setClickAttributionWindow(1);
          } on PlatformException {
            fail('PlatformException should not be rethrown');
          }
        });
      });
    });
  });
}
